use std::{collections::HashSet, sync::Arc};

use btleplug::platform::{Adapter, PeripheralId};
use tauri::{
    async_runtime::{JoinHandle, Mutex, RwLock},
    AppHandle, Runtime,
};

use crate::config_db;

pub struct AppState {
    pub db: Arc<Mutex<sled::Db>>,
    pub central_adapter: Arc<RwLock<Option<Adapter>>>,
    pub central_event_handler: Arc<Mutex<Option<JoinHandle<()>>>>,
    pub found_peripherals: Arc<RwLock<HashSet<PeripheralId>>>,
    pub scanning: Arc<Mutex<bool>>,
    pub connected_peripheral: Arc<Mutex<Option<PeripheralId>>>,
}

unsafe impl Send for AppState {}
unsafe impl Sync for AppState {}

impl AppState {
    pub async fn new<R: Runtime>(app: &AppHandle<R>) -> anyhow::Result<Self> {
        let db = config_db::open_config_db(app)?;

        Ok(Self {
            db: Arc::new(Mutex::new(db)),
            central_adapter: Arc::new(RwLock::new(None)),
            central_event_handler: Arc::new(Mutex::new(None)),
            found_peripherals: Arc::new(RwLock::new(HashSet::new())),
            scanning: Arc::new(Mutex::new(false)),
            connected_peripheral: Arc::new(Mutex::new(None)),
        })
    }

    pub async fn set_central_adapter(&self, adapter: Adapter) {
        let mut central_adapter = self.central_adapter.write().await;
        *central_adapter = Some(adapter);
    }

    pub async fn clear_central_adapter(&self) {
        let mut central_adapter = self.central_adapter.write().await;
        *central_adapter = None;
    }

    pub async fn get_central_adapter(&self) -> Option<Adapter> {
        let central_adapter = self.central_adapter.read().await;
        central_adapter.clone()
    }

    pub async fn set_central_event_handler(&self, handler: JoinHandle<()>) {
        let mut central_event_handler = self.central_event_handler.lock().await;
        *central_event_handler = Some(handler);
    }

    pub async fn clear_central_event_handler(&self) {
        let mut central_event_handler = self.central_event_handler.lock().await;
        if let Some(handler) = central_event_handler.take() {
            handler.abort();
        }
    }

    pub async fn get_found_peripherals(&self) -> HashSet<PeripheralId> {
        let found_peripherals = self.found_peripherals.read().await;
        found_peripherals.clone()
    }

    pub async fn update_found_peripherals(&self, peripheral: PeripheralId) {
        let mut found_peripherals = self.found_peripherals.write().await;
        found_peripherals.insert(peripheral);
    }

    pub async fn clear_found_peripherals(&self) {
        let mut found_peripherals = self.found_peripherals.write().await;
        found_peripherals.clear();
    }

    pub async fn set_scanning(&self, scanning_state: bool) {
        let mut scanning = self.scanning.lock().await;
        *scanning = scanning_state;
    }

    pub async fn set_connected_peripheral(&self, peripheral: PeripheralId) {
        let mut connected_peripheral = self.connected_peripheral.lock().await;
        *connected_peripheral = Some(peripheral);
    }

    pub async fn clear_connected_peripheral(&self) {
        let mut connected_peripheral = self.connected_peripheral.lock().await;
        *connected_peripheral = None;
    }

    pub async fn get_connected_peripheral(&self) -> Option<PeripheralId> {
        let connected_peripheral = self.connected_peripheral.lock().await;
        connected_peripheral.clone()
    }
}
